package SAE.ApiREST.WebService.controller;

import SAE.ApiREST.WebService.exception.TeacherException;
import SAE.ApiREST.WebService.model.Teacher;
import SAE.ApiREST.WebService.service.TeacherService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import java.util.List;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.linkTo;
import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.methodOn;

@Controller
@RequestMapping("/ProfWebService")
public class TeacherController {

    @Autowired
    private TeacherService teacherServ;


    /*public void setiTeacherServ(TeacherService iTeacherS) {
        this.teacherServ = iTeacherS;
    }*/


    @GetMapping(value = "/all", produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseStatus(HttpStatus.OK)
    public @ResponseBody CollectionModel<Teacher> getAllTeacher(){
        return CollectionModel.of(
                teacherServ.getAllTeacher(),
                linkTo(methodOn(TeacherController.class).getAllTeacher()).withSelfRel());
    }
    @PostMapping(value = "addTeacher",produces = MediaType.APPLICATION_JSON_VALUE, consumes = MediaType.APPLICATION_JSON_VALUE)
    @ResponseStatus(HttpStatus.CREATED)
    public @ResponseBody EntityModel<Teacher> createTeacher( @RequestBody Teacher teach){
        
        return EntityModel.of(teach,
                linkTo(methodOn(TeacherController.class).createTeacher(teach)).withSelfRel(),
                linkTo(methodOn(TeacherController.class).getAllTeacher()).withRel("all"));
    }
    @GetMapping(value = "/getid/{id}")
    public @ResponseBody EntityModel<Teacher> getTeachById(@PathVariable("id") Integer id){
        Teacher tt = teacherServ.getTeacherById(id);
        if( tt == null ){
            throw new TeacherException("No teacher found for this id !");
        }
        return EntityModel.of(tt,
                linkTo(methodOn(TeacherController.class).getTeachById(id)).withSelfRel(),
                linkTo(methodOn(TeacherController.class).getAllTeacher()).withRel("all"));
    }
    @GetMapping(value = "/getusername/{username}")
    public @ResponseBody EntityModel<Teacher> getTeachByUsername(@PathVariable("username") String username) {
        Teacher tt = teacherServ.getTeacherByUsername(username);
        if (tt == null) {
            throw new TeacherException("No teacher found for this username");
        }
        return EntityModel.of(tt,
                linkTo(methodOn(TeacherController.class).getTeachByUsername(username)).withSelfRel(),
                linkTo(methodOn(TeacherController.class).getAllTeacher()).withRel("all"));
    }
    @GetMapping( value = "/getmail/{mail}" )
    public @ResponseBody EntityModel<Teacher> getTeachByMail(@PathVariable("mail") String mail) {
        Teacher tt = teacherServ.getTeacherByMail(mail);
        if( tt == null ) {
            throw new TeacherException("No teacher found for this mail");
        }
        return EntityModel.of(tt,
                linkTo(methodOn(TeacherController.class).getTeachByMail(mail)).withSelfRel(),
                linkTo(methodOn(TeacherController.class).getAllTeacher()).withRel("all"));
    }
    @GetMapping( value = "/getdate/{date}" )
    public @ResponseBody Teacher getTeachByDate(@PathVariable("date") String date) {
        Teacher tt = teacherServ.getTeacherByMail(date);
        if( tt == null ) {
            throw new TeacherException("No teacher found for this mail");
        }
        return  tt;
    }
    @PutMapping( value = "/modify/{username}" )
    public @ResponseBody EntityModel<Teacher> modifyTeachUsername(@PathVariable("username") String username, Teacher tt){
        if( username == "" ){
            throw new TeacherException("Username provided for modification is empty");
        }
        teacherServ.modifyUsername(tt,username);
        return EntityModel.of(tt,
                linkTo(methodOn(TeacherController.class).modifyTeachUsername(username,tt)).withSelfRel(),
                linkTo(methodOn(TeacherController.class).getAllTeacher()).withRel("all"));
    }
    @DeleteMapping( value = "delete")
    public @ResponseBody EntityModel<List<Teacher>> deleteTeacher(Integer id){
        return EntityModel.of(teacherServ.deleteTeacher(id),
                linkTo(methodOn(TeacherController.class).deleteTeacher(id)).withSelfRel(),
                linkTo(methodOn(TeacherController.class).getAllTeacher()).withRel("all"));
    }
}
